/********************************************************************
Copyright 2017  David Edmundson <davidedmundson@kde.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/

#ifndef XWINDOWSYSTEMEVENTBATCHER_H
#define XWINDOWSYSTEMEVENTBATCHER_H

#include <QObject>

#include <KWindowSystem>
#include <QHash>

/*
 * Relay class for KWindowSystem events that batches updates
 */
class XWindowSystemEventBatcher : public QObject
{
    Q_OBJECT
public:
    XWindowSystemEventBatcher(QObject *parent);
Q_SIGNALS:
    void windowAdded(WId window);
    void windowRemoved(WId window);
    void windowChanged(WId window, NET::Properties properties, NET::Properties2 properties2);

protected:
    void timerEvent(QTimerEvent *event) override;

private:
    struct AllProps {
        NET::Properties properties = {};
        NET::Properties2 properties2 = {};
    };
    QHash<WId, AllProps> m_cache;
    int m_timerId = 0;
};

#endif
