/*
 *  Copyright 2020 David Edmundson <davidedmundson@kde.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <QDebug>
#include <QGuiApplication>

#include "../systemclipboard/systemclipboard.h"

int main(int argc, char **argv)
{
    QGuiApplication app(argc, argv);
    auto clip = SystemClipboard::instance();
    QObject::connect(clip, &SystemClipboard::changed, &app, [clip](QClipboard::Mode mode) {
        if (mode != QClipboard::Clipboard) {
            return;
        }
        auto dbg = qDebug();
        dbg << "New clipboard content: ";

        if (clip->mimeData(QClipboard::Clipboard)) {
            dbg << clip->mimeData(QClipboard::Clipboard)->text();
        } else {
            dbg << "[empty]";
        }
    });

    qDebug() << "Watching for new clipboard content...";

    app.exec();
}
