#ifndef __XML_STRINGS_H__
#define __XML_STRINGS_H__

/*
 * KFontInst - KDE Font Installer
 *
 * Copyright 2003-2009 Craig Drummond <craig@kde.org>
 *
 * ----
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#define FONTLIST_TAG "fontlist"
#define FONT_TAG "font"
#define FILE_TAG "file"
#define PATH_ATTR "path"
#define NAME_ATTR "name"
#define FOUNDRY_ATTR "foundry"
#define FAMILY_ATTR "family"
#define FAMILY_TAG "family"
#define WEIGHT_ATTR "weight"
#define WIDTH_ATTR "width"
#define SLANT_ATTR "slant"
#define SCALABLE_ATTR "scalable"
#define FACE_ATTR "face"
#define LANGS_ATTR "langs"
#define SYSTEM_ATTR "system"
#define ERROR_ATTR "error"
#define LANG_SEP ","

#endif
